@@ -8,11 +8,15 @@
 # See: https://arxiv.org/abs/1709.03856
 # TODO: move this over to TorchRankerAgent when it is ready.
 
+from typing import Optional
+from parlai.core.params import ParlaiParser
+from parlai.core.opt import Opt
 from parlai.core.agents import Agent
 from parlai.core.dict import DictionaryAgent
 from parlai.utils.misc import maintain_dialog_history, load_cands
 from parlai.core.torch_agent import TorchAgent
 from parlai.utils.io import PathManager
+import parlai.utils.torch as torch_utils
 import parlai.utils.logging as logging
 from .modules import Starspace
 
@@ -47,12 +51,14 @@ class StarspaceAgent(Agent):
     def dictionary_class():
         return DictionaryAgent
 
-    @staticmethod
-    def add_cmdline_args(argparser):
+    @classmethod
+    def add_cmdline_args(
+        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
+    ) -> ParlaiParser:
         """
         Add command-line arguments specifically for this agent.
         """
-        agent = argparser.add_argument_group('StarSpace Arguments')
+        agent = parser.add_argument_group('StarSpace Arguments')
         agent.add_argument(
             '-emb',
             '--embedding-type',
@@ -174,7 +180,8 @@ class StarspaceAgent(Agent):
             type=str,
             help='File of cands to use for prediction',
         )
-        StarspaceAgent.dictionary_class().add_cmdline_args(argparser)
+        cls.dictionary_class().add_cmdline_args(parser, partial_opt=partial_opt)
+        return parser
 
     def __init__(self, opt, shared=None):
         """
@@ -278,7 +285,7 @@ class StarspaceAgent(Agent):
         """
         Set overridable opts from loaded opt file.
 
-        Print out each added key and each overriden key. Only override args specific to
+        Print out each added key and each overridden key. Only override args specific to
         the model.
         """
         model_args = {'embeddingsize', 'optimizer'}
@@ -425,7 +432,7 @@ class StarspaceAgent(Agent):
                     xe, ye = self.model(xs, ys, self.fixedCands)
                     self.fixedX = ye
                 else:
-                    # fixed candidate embed vectors are cached, dont't recompute
+                    # fixed candidate embed vectors are cached, don't recompute
                     blah = torch.LongTensor([1])
                     xe, ye = self.model(xs, ys, [blah])
                     ye = self.fixedX
@@ -433,7 +440,7 @@ class StarspaceAgent(Agent):
                 # test set prediction uses candidates
                 xe, ye = self.model(xs, ys, cands[0])
             pred = nn.CosineSimilarity().forward(xe, ye)
-            # This is somewhat costly which we could avoid if we do not evalute ranking.
+            # This is somewhat costly which we could avoid if we do not evaluate ranking.
             # i.e. by only doing: val,ind = pred.max(0)
             val, ind = pred.sort(descending=True)
             # predict the highest scoring candidate, and return it.
@@ -550,8 +557,7 @@ class StarspaceAgent(Agent):
             data['model'] = self.model.state_dict()
             data['optimizer'] = self.optimizer.state_dict()
             data['opt'] = self.opt
-            with PathManager.open(path, 'wb') as handle:
-                torch.save(data, handle)
+            torch_utils.atomic_save(data, path)
             with PathManager.open(path + '.opt', 'w') as handle:
                 json.dump(self.opt, handle)
 
@@ -562,9 +568,10 @@ class StarspaceAgent(Agent):
         print('Loading existing model params from ' + path)
         import parlai.utils.pickle
 
-        data = torch.load(
-            path, map_location=lambda cpu, _: cpu, pickle_module=parlai.utils.pickle
-        )
+        with PathManager.open(path, 'rb') as f:
+            data = torch.load(
+                f, map_location=lambda cpu, _: cpu, pickle_module=parlai.utils.pickle
+            )
         self.model.load_state_dict(data['model'])
         self.reset()
         self.optimizer.load_state_dict(data['optimizer'])
