@@ -19,7 +19,9 @@ import requests
 import shutil
 import hashlib
 import tqdm
+import gzip
 import math
+import contextlib
 import parlai.utils.logging as logging
 from parlai.utils.io import PathManager
 
@@ -29,6 +31,17 @@ except ImportError:
     from multiprocessing import Pool
 
 
+try:
+    # internal infra requires special attention to use http sessions
+    from parlai_fb import get_http_session
+except (ImportError, AttributeError):
+
+    @contextlib.contextmanager
+    def get_http_session():
+        with requests.Session() as session:
+            yield session
+
+
 class DownloadableFile:
     """
     A class used to abstract any file that has to be downloaded online.
@@ -73,7 +86,9 @@ class DownloadableFile:
                 # remove_dir(dpath)
                 raise AssertionError(
                     f"Checksum for {self.file_name} from \n{self.url}\n"
-                    "does not match the expected checksum. Please try again."
+                    f"does not match the expected checksum:\n"
+                    f"{sha256_hash.hexdigest()} (received) != {self.hashcode} (expected)\n"
+                    f"\nPlease try again. You may need to manually delete {self.filename}."
                 )
             else:
                 logging.debug("Checksum Successful")
@@ -93,17 +108,20 @@ class DownloadableFile:
         """
         Performs a HEAD request to check if the URL / Google Drive ID is live.
         """
-        session = requests.Session()
-        if self.from_google:
-            URL = 'https://docs.google.com/uc?export=download'
-            response = session.head(URL, params={'id': self.url}, stream=True)
-        else:
-            headers = {
-                'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36'
-            }
-            response = session.head(self.url, allow_redirects=True, headers=headers)
-        status = response.status_code
-        session.close()
+        with get_http_session() as session:
+            if self.from_google:
+                URL = 'https://docs.google.com/uc?export=download'
+                response = session.head(URL, params={'id': self.url}, stream=True)
+            else:
+                headers = {
+                    'User-Agent': (
+                        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) '
+                        'AppleWebKit/537.36 (KHTML, like Gecko) '
+                        'Chrome/77.0.3865.90 Safari/537.36'
+                    )
+                }
+                response = session.head(self.url, allow_redirects=True, headers=headers)
+            status = response.status_code
 
         assert status == 200
 
@@ -164,7 +182,7 @@ def download(url, path, fname, redownload=False, num_retries=5):
     while download and retry > 0:
         response = None
 
-        with requests.Session() as session:
+        with get_http_session() as session:
             try:
                 response = session.get(url, stream=True, timeout=5)
 
@@ -234,7 +252,7 @@ def remove_dir(path):
     shutil.rmtree(path, ignore_errors=True)
 
 
-def untar(path, fname, delete=True):
+def untar(path, fname, delete=True, flatten_tar=False):
     """
     Unpack the given archive file to the same directory.
 
@@ -250,10 +268,10 @@ def untar(path, fname, delete=True):
     if ".zip" in fname:
         return _unzip(path, fname, delete=delete)
     else:
-        return _untar(path, fname, delete=delete)
+        return _untar(path, fname, delete=delete, flatten=flatten_tar)
 
 
-def _untar(path, fname, delete=True):
+def _untar(path, fname, delete=True, flatten=False):
     """
     Unpack the given archive file to the same directory.
 
@@ -280,7 +298,11 @@ def _untar(path, fname, delete=True):
                 # internal file systems will actually create a literal "."
                 # directory, so we gotta watch out for that
                 item_name = item_name[2:]
-            fn = os.path.join(path, item_name)
+            if flatten:
+                # flatten the tar file if there are subdirectories
+                fn = os.path.join(path, os.path.split(item_name)[-1])
+            else:
+                fn = os.path.join(path, item_name)
             logging.debug(f"Extracting to {fn}")
             if item.isdir():
                 PathManager.mkdirs(fn)
@@ -291,7 +313,46 @@ def _untar(path, fname, delete=True):
                 raise NotImplementedError("No support for symlinks etc. right now.")
 
     if delete:
-        PathManager.rm(fullpath)
+        try:
+            PathManager.rm(fullpath)
+        except PermissionError:
+            logging.error(
+                f"Tried to delete {fullpath} but got a permission error. This "
+                "is known to happen in Windows and is probably not fatal."
+            )
+
+
+def ungzip(path, fname, deleteGZip=True):
+    """
+    Unzips the given gzip compressed file to the same directory.
+
+    :param str path:
+        The folder containing the archive. Will contain the contents.
+
+    :param str fname:
+        The filename of the archive file.
+
+    :param bool deleteGZip:
+        If true, the compressed file will be deleted after extraction.
+    """
+
+    def _get_output_filename(input_fname):
+        GZIP_EXTENSIONS = ('.gz', '.gzip', '.tgz', '.tar')
+        for ext in GZIP_EXTENSIONS:
+            if input_fname.endswith(ext):
+                return input_fname[: -len(ext)]
+        return f'{input_fname}_unzip'
+
+    logging.debug(f'unzipping {fname}')
+    fullpath = os.path.join(path, fname)
+
+    with gzip.open(PathManager.open(fullpath, 'rb'), 'r') as fin, PathManager.open(
+        _get_output_filename(fullpath), 'wb'
+    ) as fout:
+        shutil.copyfileobj(fin, fout)
+
+    if deleteGZip:
+        os.remove(fullpath)
 
 
 def _unzip(path, fname, delete=True):
@@ -322,7 +383,13 @@ def _unzip(path, fname, delete=True):
             with zf.open(member, 'r') as inf, PathManager.open(outpath, 'wb') as outf:
                 shutil.copyfileobj(inf, outf)
     if delete:
-        PathManager.rm(fullpath)
+        try:
+            PathManager.rm(fullpath)
+        except PermissionError:
+            logging.error(
+                f"Tried to delete {fullpath} but got a permission error. This "
+                "is known to happen in Windows and is probably not fatal."
+            )
 
 
 def _get_confirm_token(response):
@@ -338,7 +405,7 @@ def download_from_google_drive(gd_id, destination):
     """
     URL = 'https://docs.google.com/uc?export=download'
 
-    with requests.Session() as session:
+    with get_http_session() as session:
         response = session.get(URL, params={'id': gd_id}, stream=True)
         token = _get_confirm_token(response)
 
@@ -360,7 +427,13 @@ def get_model_dir(datapath):
 
 
 def download_models(
-    opt, fnames, model_folder, version='v1.0', path='aws', use_model_type=False
+    opt,
+    fnames,
+    model_folder,
+    version='v1.0',
+    path='aws',
+    use_model_type=False,
+    flatten_tar=False,
 ):
     """
     Download models into the ParlAI model zoo from a url.
@@ -396,7 +469,7 @@ def download_models(
                 url = path + '/' + fname
             download(url, dpath, fname)
             if '.tgz' in fname or '.gz' in fname or '.zip' in fname:
-                untar(dpath, fname)
+                untar(dpath, fname, flatten_tar=flatten_tar)
         # Mark the data as built.
         mark_done(dpath, version)
 
@@ -439,11 +512,12 @@ def modelzoo_path(datapath, path):
                 module_name_ = 'parlai.zoo.{}'.format(animal_)
                 my_module = importlib.import_module(module_name_)
                 my_module.download(datapath)
-            except (ImportError, AttributeError):
+            except (ImportError, AttributeError) as exc:
                 # truly give up
                 raise ImportError(
                     f'Could not find pretrained model in {module_name} or {module_name_}.'
-                )
+                    ' Please check your spelling and make sure you\'ve pulled from master.'
+                ) from exc
 
         return os.path.join(datapath, 'models', model_path)
     else:
@@ -556,7 +630,7 @@ def download_multiprocess(
             error_path, 'parlai_download_multiprocess_errors_%s.log' % now
         )
 
-        with PathManager.open(os.path.join(error_filename), 'w+') as error_file:
+        with PathManager.open(os.path.join(error_filename), 'w') as error_file:
             error_file.write(json.dumps(collected_errors))
             logging.error(f'Summary of errors written to {error_filename}')
 
