@@ -467,7 +467,7 @@ class TorchAgent(ABC, Agent):
         """
         Return the dictionary class that this agent expects to use.
 
-        Can be overriden if a more complex dictionary is required.
+        Can be overridden if a more complex dictionary is required.
         """
         return DictionaryAgent
 
@@ -476,7 +476,7 @@ class TorchAgent(ABC, Agent):
         """
         Return the history class that this agent expects to use.
 
-        Can be overriden if a more complex history is required.
+        Can be overridden if a more complex history is required.
         """
         return History
 
@@ -779,7 +779,7 @@ class TorchAgent(ABC, Agent):
             self.fp16_impl = self.opt.get('fp16_impl', 'safe')
 
         if shared is None:
-            # intitialize any important structures from scratch
+            # intialize any important structures from scratch
             self.dict = self.build_dictionary()
 
             if opt.get('fp16') or opt.get('force_fp16_tokens'):
@@ -1644,7 +1644,7 @@ class TorchAgent(ABC, Agent):
         Returns a namedtuple Batch. See original definition above for in-depth
         explanation of each field.
 
-        If you want to include additonal fields in the batch, you can subclass
+        If you want to include additional fields in the batch, you can subclass
         this function and return your own "Batch" namedtuple: copy the Batch
         namedtuple at the top of this class, and then add whatever additional
         fields that you want to be able to access. You can then call
@@ -1813,7 +1813,7 @@ class TorchAgent(ABC, Agent):
         between the original history and the temporary history. If you require
         such delimiter or spacing, you should include it in the temp history.
 
-        Intentionally overrideable so more complex models can insert temporary history
+        Intentionally overridable so more complex models can insert temporary history
         strings, i.e. strings that are removed from the history after a single turn.
         """
         return observation.get('temp_history')
@@ -2306,18 +2306,18 @@ class TorchAgent(ABC, Agent):
             self._number_grad_accum = (self._number_grad_accum + 1) % update_freq
 
             # we're doing gradient accumulation, so we don't need to sync gradients
-            # amoung GPUs
+            # among GPUs
             if self._number_grad_accum != 0 and is_distributed():
                 # accumulate without syncing
                 with self.model.no_sync():
                     if self.fp16:
-                        self.optimizer.backward(loss, update_master_grads=False)
+                        self.optimizer.backward(loss, update_main_grads=False)
                     else:
                         loss.backward()
                 return
 
         if self.fp16:
-            self.optimizer.backward(loss, update_master_grads=False)
+            self.optimizer.backward(loss, update_main_grads=False)
         else:
             loss.backward()
 
@@ -2342,13 +2342,13 @@ class TorchAgent(ABC, Agent):
         if self.fp16:
             # we've been accumulating grads in fp16 and delaying the fp32 copy update.
             # finally time to perform the update.
-            self.optimizer.update_master_grads()
+            self.optimizer.update_main_grads()
 
         if self.opt.get('gradient_clip', -1) > 0 or self.fp16:
             if self.fp16:
                 # clip grad norm is where we check for fp16 overflows, so we need
                 # to do it regardless of whether gradient clipping is off
-                grad_norm = self.optimizer.clip_master_grads(self.opt['gradient_clip'])
+                grad_norm = self.optimizer.clip_main_grads(self.opt['gradient_clip'])
             else:
                 grad_norm = torch.nn.utils.clip_grad_norm_(
                     self.model.parameters(), self.opt['gradient_clip']
