@@ -21,7 +21,7 @@ import torch
 from mephisto.operations.hydra_config import register_script_config
 from mephisto.operations.operator import Operator
 from mephisto.tools.scripts import load_db_and_process_config
-from omegaconf import DictConfig
+from omegaconf import DictConfig, OmegaConf
 
 from parlai.crowdsourcing.tasks.acute_eval.analysis import (
     AcuteAnalyzer,
@@ -123,7 +123,7 @@ class FastAcuteExecutor(object):
             and self.fast_acute_args.model_pairs is None
         ):
             raise RuntimeError(
-                'Either models or model-pairs should be set for comparision.'
+                'Either models or model-pairs should be set for comparison.'
             )
         if self.fast_acute_args.model_pairs is not None:
             model_pairs = self.fast_acute_args.model_pairs.split(',')
@@ -478,6 +478,7 @@ class FastAcuteExecutor(object):
         self.set_up_acute_eval()
         db, cfg = load_db_and_process_config(self.args)
         print(f'*** RUN ID: {cfg.mephisto.task.task_name} ***')
+        print(f'\nHydra config:\n{OmegaConf.to_yaml(cfg)}')
         operator = Operator(db)
         operator.validate_and_run_config(run_config=cfg.mephisto, shared_state=None)
         operator.wait_for_runs_then_shutdown(
@@ -547,6 +548,7 @@ class FastAcuteExecutor(object):
 
 
 defaults = [
+    '_self_',
     {"mephisto/blueprint": FAST_ACUTE_BLUEPRINT_TYPE},
     {"mephisto/architect": "local"},
     {"mephisto/provider": "mock"},
@@ -570,7 +572,7 @@ class TestScriptConfig(MTurkRunScriptConfig):
 register_script_config(name='fast_acute_scriptconfig', module=TestScriptConfig)
 
 
-@hydra.main(config_name="fast_acute_scriptconfig")
+@hydra.main(config_path="hydra_configs", config_name="fast_acute_scriptconfig")
 def main(cfg: DictConfig) -> None:
 
     runner = FastAcuteExecutor(cfg)
