@@ -9,6 +9,10 @@ Code for LR Schedulers.
 See ParlAILRScheduler (super class) and subclasses for detailed documentation
 """
 
+import math
+from typing import Optional
+from parlai.core.params import ParlaiParser
+from parlai.core.opt import Opt
 from abc import abstractmethod
 from torch import optim
 import numpy as np
@@ -55,7 +59,7 @@ class ParlAILRScheduler(object):
     def _init_warmup_scheduler(self, optimizer, states):
         updates_so_far = states.get('number_training_updates', 0)
         if self.warmup_updates > 0 and (
-            updates_so_far <= self.warmup_updates or self.hard_reset
+            updates_so_far < self.warmup_updates or self.hard_reset
         ):
             self.warmup_scheduler = optim.lr_scheduler.LambdaLR(
                 optimizer, self._warmup_lr
@@ -82,7 +86,7 @@ class ParlAILRScheduler(object):
         return (
             hasattr(self, 'warmup_scheduler')
             and self.warmup_scheduler is not None
-            and self._number_training_updates <= self.warmup_updates
+            and self._number_training_updates < self.warmup_updates
         )
 
     def _warmup_lr(self, step):
@@ -106,7 +110,10 @@ class ParlAILRScheduler(object):
         self._number_training_updates = states.get('number_training_updates', 0)
 
         try:
-            self.scheduler.get_last_lr()
+            if self._is_lr_warming_up():
+                self.warmup_scheduler.get_last_lr()
+            else:
+                self.scheduler.get_last_lr()
         except AttributeError:
             # on older pytorches
             self.step(self._number_training_updates)
@@ -129,8 +136,10 @@ class ParlAILRScheduler(object):
         return self.warmup_scheduler.state_dict()
 
     @classmethod
-    def add_cmdline_args(cls, argparser):
-        lr_group = argparser.add_argument_group('Learning Rate Scheduler')
+    def add_cmdline_args(
+        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
+    ) -> ParlaiParser:
+        lr_group = parser.add_argument_group('Learning Rate Scheduler')
         lr_group.add_argument(
             '--lr-scheduler',
             type=str,
@@ -152,14 +161,6 @@ class ParlAILRScheduler(object):
             help='Decay factor for LR scheduler, or how much LR is multiplied by '
             'when it is lowered.',
         )
-        lr_group.add_argument(
-            '--max-lr-steps',
-            type=int,
-            default=-1,
-            help='Number of train steps the scheduler should take after warmup. '
-            'Training is terminated after this many steps. This should only be '
-            'set for --lr-scheduler cosine or linear',
-        )
         lr_group.add_argument(
             '--invsqrt-lr-decay-gamma',
             type=int,
@@ -191,6 +192,7 @@ class ParlAILRScheduler(object):
             hidden=True,
             help='Accumulate gradients N times before performing an optimizer.step().',
         )
+        return parser
 
     @classmethod
     def lr_scheduler_factory(cls, opt, optimizer, states, hard_reset=False):
@@ -216,7 +218,11 @@ class ParlAILRScheduler(object):
         decay = opt.get('lr_scheduler_decay', 0.5)
         warmup_updates = opt.get('warmup_updates', -1)
         warmup_rate = opt.get('warmup_rate', 1e-4)
-        max_lr_steps = opt.get('max_lr_steps', -1)
+        max_lr_steps = opt.get('max_train_steps', -1)
+        if opt.get('max_lr_steps', -1) > 0:
+            raise ValueError(
+                '--max-lr-steps is **DEPRECATED**; please set --max-train-steps directly'
+            )
         invsqrt_lr_decay_gamma = opt.get('invsqrt_lr_decay_gamma', -1)
 
         if opt.get('lr_scheduler') == 'none':
@@ -411,7 +417,8 @@ class InvSqrtLRScheduler(ParlAILRScheduler):
         When steps taken == invsqrt_lr_decay_gamma, the lr multiplier is 1
         """
         super().__init__(hard_reset, warmup_updates, warmup_rate)
-        self.max_lr_steps = max_lr_steps
+        assert self.warmup_updates >= 0
+        self.max_lr_steps = max_lr_steps - self.warmup_updates
         self.invsqrt_lr_decay_gamma = invsqrt_lr_decay_gamma
         if invsqrt_lr_decay_gamma <= 0:
             warn_once(
@@ -460,9 +467,13 @@ class CosineLRScheduler(ParlAILRScheduler):
         """
         super().__init__(hard_reset, warmup_updates, warmup_rate)
         if max_lr_steps <= 0:
-            raise ValueError('--lr-scheduler cosine requires setting --max-lr-steps')
-        self.max_lr_steps = max_lr_steps
-        self.scheduler = optim.lr_scheduler.CosineAnnealingLR(optimizer, max_lr_steps)
+            raise ValueError('--lr-scheduler cosine requires setting --max-train-steps')
+        assert self.warmup_updates >= 0
+        self.max_lr_steps = max_lr_steps - self.warmup_updates
+        self.scheduler = optim.lr_scheduler.LambdaLR(optimizer, self._cosine_lr)
+
+    def _cosine_lr(self, step):
+        return math.cos(math.pi * step / (2 * self.max_lr_steps))
 
     def train_step(self, scheduler_steps):
         if scheduler_steps >= self.max_lr_steps:
@@ -495,14 +506,14 @@ class LinearLRScheduler(ParlAILRScheduler):
         """
         super().__init__(hard_reset, warmup_updates, warmup_rate)
         if max_lr_steps <= 0:
-            raise ValueError('--lr-scheduler linear requires setting --max-lr-steps')
-        self.max_lr_steps = max_lr_steps
+            raise ValueError('--lr-scheduler linear requires setting --max-train-steps')
+        assert self.warmup_updates >= 0
+        self.max_lr_steps = max_lr_steps - self.warmup_updates
         self.scheduler = optim.lr_scheduler.LambdaLR(optimizer, self._linear_lr)
 
     def _linear_lr(self, step):
         # this multiplicative factor ensures linear decay rate
-        # lr_mult = float(self.max_lr_steps - step - 1) / float(self.max_lr_steps - step)
-        lr_mult = max(0.0, 1e-6 + (1.0 - step / self.max_lr_steps) * (1 - 1e-6))
+        lr_mult = max(0.0, 1.0 - step / self.max_lr_steps)
         return lr_mult
 
     def train_step(self, scheduler_steps):
