@@ -9,7 +9,7 @@ Talk with a model using a web UI.
 ## Examples
 
 ```shell
-parlai interactive_web -mf "zoo:tutorial_transformer_generator/model"
+parlai interactive_web --model-file "zoo:tutorial_transformer_generator/model"
 ```
 """
 
@@ -23,6 +23,7 @@ from parlai.core.script import ParlaiScript, register_script
 import parlai.utils.logging as logging
 
 import json
+import time
 
 HOST_NAME = 'localhost'
 PORT = 8080
@@ -194,11 +195,10 @@ class MyHandler(BaseHTTPRequestHandler):
             model_response = self._interactive_running(
                 SHARED.get('opt'), body.decode('utf-8')
             )
-
             self.send_response(200)
             self.send_header('Content-type', 'application/json')
             self.end_headers()
-            json_str = json.dumps(model_response)
+            json_str = json.dumps(model_response.json_safe_payload())
             self.wfile.write(bytes(json_str, 'utf-8'))
         elif self.path == '/reset':
             self.send_response(200)
@@ -250,12 +250,26 @@ def setup_interweb_args(shared):
     return parser
 
 
-def interactive_web(opt):
+def shutdown():
+    global SHARED
+    if 'server' in SHARED:
+        SHARED['server'].shutdown()
+    SHARED.clear()
 
-    SHARED['opt']['task'] = 'parlai.agents.local_human.local_human:LocalHumanAgent'
+
+def wait():
+    global SHARED
+    while not SHARED.get('ready'):
+        time.sleep(0.01)
+
+
+def interactive_web(opt):
+    global SHARED
+
+    opt['task'] = 'parlai.agents.local_human.local_human:LocalHumanAgent'
 
     # Create model and assign it to the specified task
-    agent = create_agent(SHARED.get('opt'), requireModelExists=True)
+    agent = create_agent(opt, requireModelExists=True)
     agent.opt.log()
     SHARED['opt'] = agent.opt
     SHARED['agent'] = agent
@@ -263,9 +277,11 @@ def interactive_web(opt):
 
     MyHandler.protocol_version = 'HTTP/1.0'
     httpd = HTTPServer((opt['host'], opt['port']), MyHandler)
+    SHARED['server'] = httpd
     logging.info('http://{}:{}/'.format(opt['host'], opt['port']))
 
     try:
+        SHARED['ready'] = True
         httpd.serve_forever()
     except KeyboardInterrupt:
         pass
