@@ -4,6 +4,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from parlai.core.params import ParlaiParser
 import copy
 import json
 import os
@@ -122,12 +123,14 @@ class ConvAI2PersonaTopicifierTeacher(Convai2DefaultTeacher):
 
     def __init__(self, opt, shared=None):
         if 'stream' in opt['datatype']:
-            warn_once('Warning: this teacher is not compatible with StreamDialogData!')
+            warn_once(
+                'Warning: the BST Convai2 teacher is not compatible with '
+                'streaming datatypes. Switching to nonstreaming.'
+            )
             # StreamDialogData works by reading directly from a text file without any
             # alteration, but this teacher must append a WoW topic string to the context
             # of the first example of each episode.
-            assert opt['datatype'].endswith(':stream')
-            opt['datatype'] = opt['datatype'][: -len(':stream')]
+            opt['datatype'] = opt['datatype'].replace(':stream', '')
         self.persona_topicifier = PersonaTopicifier(
             opt=opt, should_have_personas=True, should_have_topics=False
         )
@@ -168,15 +171,18 @@ class EDPersonaTopicifierTeacher(EmpatheticDialoguesTeacher):
     RECOMPILE_DEFAULT = False
 
     @classmethod
-    def add_cmdline_args(cls, argparser):
-        EmpatheticDialoguesTeacher.add_cmdline_args(argparser)
-        agent = argparser.add_argument_group('EDPersonaTopicifierTeacher arguments')
+    def add_cmdline_args(
+        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
+    ) -> ParlaiParser:
+        super().add_cmdline_args(parser, partial_opt=partial_opt)
+        agent = parser.add_argument_group('EDPersonaTopicifierTeacher arguments')
         agent.add_argument(
             '--recompile-persona-topic-data',
             type='bool',
             default=cls.RECOMPILE_DEFAULT,
             help='Re-compile data with ConvAI2 personas and WoW topics added. Only useful for demonstrating how data was produced.',
         )
+        return parser
 
     def __init__(self, opt, shared=None):
         self.persona_topicifier = PersonaTopicifier(
@@ -457,7 +463,7 @@ class ContextGenerator:
 
     def __init__(self, opt, datatype: str = 'train', seed: Optional[int] = None):
         """
-        Initalize the context generator.
+        Initialize the context generator.
 
         opt: only a 'datapath' key is required, to specify the ParlAI data folder
         """
