@@ -4,6 +4,9 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from typing import Optional
+from parlai.core.params import ParlaiParser
+from parlai.core.opt import Opt
 from parlai.core.teachers import FixedDialogTeacher
 from parlai.utils.io import PathManager
 from .build import build
@@ -23,15 +26,19 @@ class ELI5Teacher(FixedDialogTeacher):
         self.messages = self.load_eli5(opt)
         self.reset()
 
-    @staticmethod
-    def add_cmdline_args(argparser):
-        group = argparser.add_argument_group('ELI5 Knowledge arguments')
+    @classmethod
+    def add_cmdline_args(
+        cls, parser: ParlaiParser, partial_opt: Optional[Opt] = None
+    ) -> ParlaiParser:
+        super().add_cmdline_args(parser, partial_opt)
+        group = parser.add_argument_group('ELI5 Knowledge arguments')
         group.add_argument(
             '--knowledge',
             type='bool',
             default=True,
             help='Whether to include supporting document knowledge',
         )
+        return parser
 
     def load_eli5(self, opt):
         """
@@ -44,7 +51,7 @@ class ELI5Teacher(FixedDialogTeacher):
         if not PathManager.exists(fname):
             raise FileNotFoundError(
                 f"{fname} not found. Please follow the instructions found at "
-                "https://github.com/facebookresearch/ParlAI/tree/master/parlai/tasks/eli5/README.md"
+                "https://github.com/facebookresearch/ParlAI/tree/main/parlai/tasks/eli5/README.md"
                 " to construct the dataset."
             )
         opt['datafile'] = fname
