@@ -18,7 +18,15 @@ from parlai.core.teachers import DialogTeacher
 from parlai.utils.data import DatatypeHelper
 import parlai.utils.logging as logging
 import parlai.tasks.wizard_of_internet.constants as CONST
+from parlai.core.mutators import register_mutator
+from parlai.tasks.wizard_of_wikipedia.agents import (
+    AddLabel as AddLabelWizWiki,
+    AddLabelLM as AddLabelLMWizWiki,
+    CheckedSentenceAsLabel as CheckedSentenceAsLabelWizWiki,
+    AddCheckedSentence as AddCheckedSentenceWizWiki,
+)
 
+import random
 from .build import build
 
 
@@ -67,6 +75,7 @@ def parse_wizard_message(message_dict, doc_lines_delim):
             CONST.SELECTED_DOCS_TITLES: [],
             CONST.SELECTED_SENTENCES: [],
             CONST.RETRIEVED_DOCS_URLS: [],
+            CONST.RETRIEVED_DOCS_TITLES: [],
         }
         docs = msg_d[CONST.CONTEXT][CONST.CONTENTS]
         selections = msg_d[CONST.CONTEXT][CONST.SELECTED_CONTENTS]
@@ -85,6 +94,7 @@ def parse_wizard_message(message_dict, doc_lines_delim):
                     knowledge[CONST.SELECTED_SENTENCES].append(line)
             full_doc = doc_lines_delim.join(doc_lines)
             knowledge[CONST.RETRIEVED_DOCS].append(full_doc)
+            knowledge[CONST.RETRIEVED_DOCS_TITLES].append(doc['title'])
             knowledge[CONST.RETRIEVED_DOCS_URLS].append(doc['url'])
             if doc_selected:
                 knowledge[CONST.SELECTED_DOCS_TITLES].append(doc['title'])
@@ -176,7 +186,7 @@ class WizardOfInternetBaseTeacher(DialogTeacher):
             d = {CONST.PERSONA: persona, CONST.TOTAL_CONVERSATION_INDEX: msg_ind}
             action = message[CONST.ACTION]
 
-            # Seperating the actions
+            # Separating the actions
             if action == CONST.ACTION_APPRENTICE_TO_WIZARD:
                 d.update(parse_apprentice_message(message))
             elif action == CONST.ACTION_WIZARD_TO_APPRENTICE:
@@ -386,8 +396,10 @@ class WizardDialogTeacher(WizardOfInternetBaseTeacher):
         for item_key in (
             CONST.RETRIEVED_DOCS,
             CONST.RETRIEVED_DOCS_URLS,
+            CONST.RETRIEVED_DOCS_TITLES,
             CONST.SELECTED_DOCS,
             CONST.SELECTED_SENTENCES,
+            CONST.SELECTED_DOCS_TITLES,
             CONST.SEARCH_QUERY,
         ):
             parlai_message[item_key] = action[item_key]
@@ -567,3 +579,62 @@ class GoldDocsTeacher(BaseKnowledgeTeacher):
 class GoldDocTitlesTeacher(BaseKnowledgeTeacher):
     def _knowledge_piece(self):
         return CONST.SELECTED_DOCS_TITLES
+
+
+@register_mutator("add_checked_sentence_to_input_woi")
+class AddCheckedSentence(AddCheckedSentenceWizWiki):
+    """
+    Adds the checked sentence to the end of the text.
+
+    E.g. run with: parlai display_data -t wizard_of_internet -n 100 -dt valid --mutators
+    flatten,add_checked_sentence_to_input_woi
+    """
+
+    @property
+    def checked_sentence_kword(self):
+        return CONST.SELECTED_SENTENCES
+
+
+@register_mutator("checked_sentence_as_label_woi")
+class CheckedSentenceAsLabel(CheckedSentenceAsLabelWizWiki):
+    """
+    Uses the checked sentence (knowledge) as label.
+
+    E.g. run with: parlai display_data -t wizard_of_internet -n 100 -dt valid --mutators
+    flatten,checked_sentence_as_label_woi
+    """
+
+    @property
+    def checked_sentence_kword(self):
+        return CONST.SELECTED_SENTENCES
+
+
+@register_mutator("add_label_to_input_woi")
+class AddLabel(AddLabelWizWiki):
+    """
+    Adds the dialogue sentence to the input.
+
+    E.g. run with: parlai display_data -t wizard_of_internet -n 100 -dt valid --mutators
+    flatten,checked_sentence_as_label_woi,add_label_to_input_woi
+    """
+
+    pass
+
+
+@register_mutator("add_label_to_input_lm_woi")
+class AddLabelLM(AddLabelLMWizWiki):
+    """
+    Adds the dialogue sentence to the input (language modeling version).
+
+    Language modeling version where a random piece of the label is sampled in
+    the input. The rest is placed inside special tokens.
+
+    E.g. run with: parlai display_data -t wizard_of_internet -n 100 -dt valid --mutators
+    flatten,add_label_to_input_lm_woi
+
+    To add the checked sentence as the label, use:
+        parlai display_data -t wizard_of_internet -n 100 -dt valid --mutators
+        flatten,add_label_to_input_lm_woi,checked_sentence_as_label_woi
+    """
+
+    pass
