@@ -1,11 +1,12 @@
 #!/usr/bin/env python3
-
 # Copyright (c) Facebook, Inc. and its affiliates.
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+import os
 import sys
 import logging
+from logging import getLogger  # noqa: F401
 
 try:
     import coloredlogs
@@ -36,7 +37,7 @@ LOGFILE_DATE_FORMAT = None
 
 COLORED_LEVEL_STYLES = {
     'spam': {'color': 'white', 'faint': True},
-    'debug': {'faint': True},
+    'debug': {'color': 'green', 'faint': True},
     'verbose': {'color': 'blue'},
     'error': {'color': 'red'},
     'info': {},
@@ -48,6 +49,8 @@ COLORED_LEVEL_STYLES = {
 
 
 def _is_interactive():
+    if os.environ.get('PARLAI_FORCE_COLOR'):
+        return True
     try:
         __IPYTHON__
         return True
@@ -72,19 +75,20 @@ class ParlaiLogger(logging.Logger):
         self.streamHandler = logging.StreamHandler(sys.stdout)
         # Log to stdout levels: console_level and above
         self.prefix = None
+        self.interactive = _is_interactive()
         self.streamHandler.setFormatter(self._build_formatter())
         super().addHandler(self.streamHandler)
 
     def _build_formatter(self):
         prefix_format = f'{self.prefix} ' if self.prefix else ''
-        if COLORED_LOGS and _is_interactive():
+        if COLORED_LOGS and self.interactive:
             return coloredlogs.ColoredFormatter(
                 prefix_format + COLORED_FORMAT,
                 datefmt=CONSOLE_DATE_FORMAT,
                 level_styles=COLORED_LEVEL_STYLES,
                 field_styles={},
             )
-        elif _is_interactive():
+        elif self.interactive:
             return logging.Formatter(
                 prefix_format + CONSOLE_FORMAT, datefmt=CONSOLE_DATE_FORMAT
             )
@@ -93,6 +97,10 @@ class ParlaiLogger(logging.Logger):
                 prefix_format + LOGFILE_FORMAT, datefmt=LOGFILE_DATE_FORMAT
             )
 
+    def force_interactive(self):
+        self.interactive = True
+        self.streamHandler.setFormatter(self._build_formatter())
+
     def log(self, msg, level=INFO):
         """
         Default Logging function.
@@ -125,7 +133,7 @@ class ParlaiLogger(logging.Logger):
 # -----------------------------------
 # Forming the logger                #
 # -----------------------------------
-logger = ParlaiLogger(name=__name__)
+logger = ParlaiLogger(name="parlai")
 
 
 def set_log_level(level):
@@ -173,11 +181,11 @@ def error(*args, **kwargs):
 
 
 def warn(*args, **kwargs):
-    return logger.warn(*args, **kwargs)
+    return logger.warning(*args, **kwargs)
 
 
 def warning(*args, **kwargs):
-    return logger.warn(*args, **kwargs)
+    return logger.warning(*args, **kwargs)
 
 
 def get_all_levels():
