@@ -16,6 +16,7 @@ import time
 import re
 import shutil
 import json
+import os
 
 from parlai.core.message import Message
 from parlai.utils.strings import colorize
@@ -31,7 +32,7 @@ except ImportError:
     __TORCH_AVAILABLE = False
 
 
-DISPLAY_MESSAGE_DEFAULT_FIELDS = {
+SPECIAL_FORMATED_DISPLAY_MESSAGE_FIELDS = {
     'episode_done',
     'id',
     'image',
@@ -41,12 +42,12 @@ DISPLAY_MESSAGE_DEFAULT_FIELDS = {
     'label_candidates',
     'text_candidates',
     'reward',
-    'eval_labels_vec',
-    'text_vec',
-    'label_candidates_vecs',
     'token_losses',
+    'metrics',
 }
 
+MUST_SHOW_MESSAGE_FIELDS = {'image', 'text', 'labels', 'eval_labels', 'reward'}
+
 
 def maintain_dialog_history(
     history,
@@ -291,24 +292,19 @@ class AttrDict(dict):
         self.__dict__ = self
 
 
-class NoLock(object):
+class SimpleCounter:
     """
-    Empty `lock`.
-
-    Does nothing when you enter or exit.
+    Simple counter object.
     """
 
-    def __enter__(self):
-        """
-        No-op.
-        """
-        return self
+    def __init__(self, value=0):
+        self.val = value
 
-    def __exit__(self, exc_type, exc_value, exc_traceback):
-        """
-        No-op.
-        """
-        pass
+    def increment(self, value=1):
+        self.val += value
+
+    def value(self):
+        return self.val
 
 
 def _report_sort_key(report_key: str) -> Tuple[str, str]:
@@ -357,6 +353,11 @@ def float_formatter(f: Union[float, int]) -> str:
 
 
 def _line_width():
+    if os.environ.get('PARLAI_FORCE_WIDTH'):
+        try:
+            return int(os.environ['PARLAI_FORCE_WIDTH'])
+        except ValueError:
+            pass
     try:
         # if we're in an interactive ipython notebook, hardcode a longer width
         __IPYTHON__
@@ -410,17 +411,12 @@ def nice_report(report) -> str:
         df = pd.DataFrame([output])
         df.columns = pd.MultiIndex.from_tuples(df.columns)
         df = df.stack().transpose().droplevel(0, axis=1)
-        result = (
-            "   "
-            + df.to_string(
-                na_rep="",
-                line_width=line_width - 3,  # -3 for the extra spaces we add
-                float_format=float_formatter,
-                index=df.shape[0] > 1,
-            )
-            .replace("\n\n", "\n")
-            .replace("\n", "\n   ")
-        )
+        result = "   " + df.to_string(
+            na_rep="",
+            line_width=line_width - 3,  # -3 for the extra spaces we add
+            float_format=float_formatter,
+            index=df.shape[0] > 1,
+        ).replace("\n\n", "\n").replace("\n", "\n   ")
         result = re.sub(r"\s+$", "", result)
         return result
     else:
@@ -458,16 +454,6 @@ def round_sigfigs(x: Union[float, 'torch.Tensor'], sigfigs=4) -> float:
             raise ex
 
 
-single_nolock = NoLock()
-
-
-def no_lock():
-    """
-    Build a nolock for other classes to use for no-op locking.
-    """
-    return single_nolock
-
-
 def clip_text(text, max_len):
     """
     Clip text to max length, adding ellipses.
@@ -505,19 +491,20 @@ def _ellipse(lst: List[str], max_display: int = 5, sep: str = '|') -> str:
 def display_messages(
     msgs: List[Dict[str, Any]],
     prettify: bool = False,
-    ignore_fields: str = '',
+    ignore_agent_reply: bool = False,
+    add_fields: str = '',
     max_len: int = 1000,
     verbose: bool = False,
 ) -> Optional[str]:
     """
     Return a string describing the set of messages provided.
 
-    If prettify is true, candidates are displayed using prettytable. ignore_fields
-    provides a list of fields in the msgs which should not be displayed.
+    If prettify is true, candidates are displayed using prettytable. add_fields provides
+    a list of fields in the msgs which should be displayed if verbose is off.
     """
 
     def _token_losses_line(
-        msg: Dict[str, Any], ignore_fields: List[str], space: str
+        msg: Dict[str, Any], fields_to_show: List[str], space: str
     ) -> Optional[str]:
         """
         Displays the loss associated with each token. Can be used for debugging
@@ -527,25 +514,28 @@ def display_messages(
         """
         key = 'token_losses'
         token_losses = msg.get(key, None)
-        if key in ignore_fields or not token_losses:
+        if key not in fields_to_show or not token_losses:
             return None
         # Reduce losses to 4 significant figures
         formatted_tl = ' | '.join(
             [f"{tl[0]} {float('{:.4g}'.format(tl[1]))}" for tl in token_losses]
         )
-        return f'{space}[{key}]: {formatted_tl}'
+        return _pretty_lines(space, key, formatted_tl, 'text2')
+
+    def _pretty_lines(indent_space, field, value, style):
+        line = '{}{} {}'.format(
+            indent_space, colorize('[' + field + ']:', 'field'), colorize(value, style)
+        )
+        return line
 
     lines = []
     episode_done = False
-    ignore_fields_ = ignore_fields.split(',')
+    extra_add_fields_ = add_fields.split(',')
     for index, msg in enumerate(msgs):
-        if msg is None or (index == 1 and 'agent_reply' in ignore_fields_):
+        if msg is None or (index == 1 and ignore_agent_reply):
             # We only display the first agent (typically the teacher) if we
             # are ignoring the agent reply.
             continue
-        agent_id = msg.get('id', '[no id field]')
-        if verbose:
-            lines.append(colorize('[id]:', 'field') + ' ' + colorize(agent_id, 'id'))
 
         if msg.get('episode_done'):
             episode_done = True
@@ -553,47 +543,76 @@ def display_messages(
         space = ''
         if len(msgs) == 2 and index == 1:
             space = '   '
+
+        agent_id = msg.get('id', '[no id field]')
+        if verbose:
+            line = _pretty_lines(
+                indent_space=space, field='id', value=agent_id, style='id'
+            )
+            lines.append(line)
+
         # Only display rewards !=0 as they are confusing in non-RL tasks.
         if msg.get('reward', 0) != 0:
             lines.append(space + '[reward: {r}]'.format(r=msg['reward']))
-        for key in msg:
-            if key not in DISPLAY_MESSAGE_DEFAULT_FIELDS and key not in ignore_fields_:
-                field = colorize('[' + key + ']:', 'field')
-                if type(msg[key]) is list:
-                    value = _ellipse(msg[key], sep='\n  ')
+
+        fields_to_show = []
+        if verbose:
+            fields_to_show = [field for field in msg]
+        else:
+            fields_to_show = [
+                field
+                for field in msg
+                if field in list(MUST_SHOW_MESSAGE_FIELDS) + extra_add_fields_
+            ]
+        fields_to_show.sort()
+
+        # Display fields without special format
+        for field in fields_to_show:
+            if field not in SPECIAL_FORMATED_DISPLAY_MESSAGE_FIELDS:
+                if type(msg[field]) is list:
+                    value = _ellipse(msg[field], sep='\n  ')
                 else:
-                    value = clip_text(str(msg.get(key)), max_len)
-                line = field + ' ' + colorize(value, 'text2')
-                lines.append(space + line)
+                    value = clip_text(str(msg.get(field)), max_len)
+                line = _pretty_lines(
+                    indent_space=space, field=field, value=value, style='text2'
+                )
+                lines.append(line)
+
+        # Display fields WITH special format requirements
+        # Display Image
         if type(msg.get('image')) in [str, torch.Tensor]:
             lines.append(f'[ image ]: {msg["image"]}')
+        # Display Text
         if msg.get('text', ''):
-            text = clip_text(msg['text'], max_len)
-            if index == 0:
-                style = 'bold_text'
-            else:
-                style = 'labels'
-            if verbose:
-                lines.append(
-                    space + colorize('[text]:', 'field') + ' ' + colorize(text, style)
-                )
-            else:
-                lines.append(
-                    space
-                    + colorize("[" + agent_id + "]:", 'field')
-                    + ' '
-                    + colorize(text, style)
-                )
+            value = clip_text(msg['text'], max_len)
+            style = 'bold_text' if index == 0 else 'labels'
+            field = 'text' if verbose else agent_id
+            line = _pretty_lines(
+                indent_space=space, field=field, value=value, style=style
+            )
+            lines.append(line)
+        # Display Label Fields
         for field in {'labels', 'eval_labels', 'label_candidates', 'text_candidates'}:
-            if msg.get(field) and field not in ignore_fields_:
-                string = '{}{} {}'.format(
-                    space,
-                    colorize('[' + field + ']:', 'field'),
-                    colorize(_ellipse(msg[field]), field),
+            if msg.get(field) and field in fields_to_show:
+                line = _pretty_lines(
+                    indent_space=space,
+                    field=field,
+                    value=_ellipse(msg[field]),
+                    style=field,
                 )
-                lines.append(string)
+                lines.append(line)
+        if msg.get('metrics') and verbose:
+            lines.append(
+                _pretty_lines(
+                    indent_space=space,
+                    field='metrics',
+                    value="\n" + nice_report(msg['metrics']),
+                    style='text',
+                )
+            )
+
         # Handling this separately since we need to clean up the raw output before displaying.
-        token_loss_line = _token_losses_line(msg, ignore_fields_, space)
+        token_loss_line = _token_losses_line(msg, fields_to_show, space)
         if token_loss_line:
             lines.append(token_loss_line)
 
@@ -626,8 +645,9 @@ def str_to_msg(txt, ignore_fields=''):
 
     def tolist(txt):
         vals = txt.split('|')
-        for v in vals:
+        for i, v in enumerate(vals):
             v = tostr(v)
+            vals[i] = v
         return vals
 
     def convert(key, value):
@@ -640,6 +660,11 @@ def str_to_msg(txt, ignore_fields=''):
             or key == 'text_candidates'
         ):
             return tolist(value)
+        elif key == 'reward':
+            try:
+                return int(value)
+            except ValueError:
+                return float(value)
         elif key == 'episode_done':
             return bool(value)
         else:
@@ -746,7 +771,7 @@ def warn_once(msg: str) -> None:
     global _seen_logs
     if msg not in _seen_logs:
         _seen_logs.add(msg)
-        logging.warn(msg)
+        logging.warning(msg)
 
 
 def error_once(msg: str) -> None:
